import React, { Component } from 'react';
import { LPInbox, LPInboxMessage } from '@brandingbrand/react-native-leanplum';

import AutoHeightImage from './AutoHeightImage';
import {
  Dimensions,
  FlatList,
  Image,
  ImageURISource,
  Modal,
  Platform,
  StyleSheet,
  Text,
  TextStyle,
  TouchableOpacity,
  View,
  ViewStyle,
  WebView,
  Linking,
  ListRenderItemInfo
} from 'react-native';

import moment from 'moment';

const APPBAR_HEIGHT = Platform.OS === 'ios' ? 44 : 56;
const STATUSBAR_HEIGHT = Platform.OS === 'ios' ? 20 : 0;

export interface LeanplumInboxPropTypes {
  logo: ImageURISource;
  appName: string;
  fontName?: string;
  buttonStyle?: ViewStyle;
  buttonTextStyle?: TextStyle;
  messageTitleStyle?: TextStyle;
}

export interface LeanplumInboxStateTypes {
  messages: any[];
  webviewVisible: boolean;
  webviewActionUrl: string;
  webviewActionTitle: string;
}

const styles = StyleSheet.create({
  ModalTitle: {
    flex: 1,
    textAlign: 'center',
    paddingLeft: 38,
    paddingTop: Platform.OS === 'ios' ? 20 : 10,
    fontSize: 16
  },
  header: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    padding: 10,
    paddingRight: 45,
    height: APPBAR_HEIGHT + STATUSBAR_HEIGHT,
    backgroundColor: Platform.OS === 'ios' ? '#EFEFF2' : '#FFF',
    borderBottomColor: 'rgba(0, 0, 0, .15)',
    borderBottomWidth: 1
  },
  hideDefaultHeader: {
    height: 0,
    padding: 0,
    overflow: 'hidden',
    marginTop: 0
  },
  closeButton: {
    width: 35,
    height: 35,
    position: 'absolute',
    bottom: 5,
    left: 10
  },
  closeButtonIcon: {
    width: 32,
    height: 32,
    paddingTop: 15
  },
  closeButtonLeft: {
    width: 32,
    height: 1,
    transform: [{ rotate: '45deg' }],
    backgroundColor: '#555'
  },
  closeButtonRight: {
    width: 32,
    height: 1,
    transform: [{ rotate: '135deg' }],
    backgroundColor: '#555'
  },
  container: {
    backgroundColor: 'white',
    flex: 1
  },
  fontSize14: {
    fontSize: 14
  },
  fontSize10: {
    fontSize: 10
  },
  messageContainer: {
    marginBottom: 10,
    backgroundColor: 'white'
  },
  messageLogoImage: {
    width: 30,
    height: 30
  },
  messageLogoView: {
    paddingLeft: 10,
    paddingTop: 10
  },
  messageSubtitle: {
    marginLeft: 10,
    marginRight: 10,
    marginBottom: 10
  },
  messageInfo: {
    flexDirection: 'row',
    alignItems: 'center',
    marginLeft: 10,
    marginRight: 10,
    marginBottom: 10
  },
  buttonContainer: {
    alignItems: 'center'
  },
  button: {
    width: 200,
    height: 40,
    justifyContent: 'center',
    alignItems: 'center',
    backgroundColor: 'red'
  },
  buttonText: {
    color: 'white',
    lineHeight: 16,
    fontSize: 16
  }
});

export default class leanplumInbox extends Component<
  LeanplumInboxPropTypes,
  LeanplumInboxStateTypes
> {
  constructor(props: LeanplumInboxPropTypes) {
    super(props);
    this.state = {
      messages: [],
      webviewVisible: false,
      webviewActionUrl: null,
      webviewActionTitle: null
    };
  }

  componentDidMount() {
    LPInbox.allMessages().then(msgs => {
      this.setState({
        messages: this.sortMessageByTime(msgs)
      });
    });
  }

  sortMessageByTime = (msgs: LPInboxMessage[]) => {
    if (!msgs || !msgs.length) {
      return msgs;
    }
    msgs.sort(function(a, b) {
      return b.deliveryTimestamp.getTime() - a.deliveryTimestamp.getTime();
    });
    return msgs;
  };

  render() {
    return (
      <View style={styles.container}>
        <FlatList
          keyExtractor={(item, index) => index + ''}
          data={this.state.messages}
          renderItem={this.renderMessage}
        />
        <Modal
          visible={this.state.webviewVisible}
          animationType={'slide'}
          onRequestClose={this.closeModal}
        >
          {this.renderModalHeader()}
          <WebView source={{ uri: this.state.webviewActionUrl }} />
        </Modal>
      </View>
    );
  }

  closeModal = () => {
    this.setState({
      webviewVisible: false,
      webviewActionUrl: null,
      webviewActionTitle: null
    });
  };

  getFontName = () => {
    return this.props.fontName ? { fontFamily: this.props.fontName } : null;
  };

  getMessageTitleStyle = () => {
    return this.props.messageTitleStyle;
  };

  renderModalHeader = () => {
    return (
      <View style={styles.header}>
        <Text style={[styles.ModalTitle, this.getFontName()]}>
          {this.state.webviewActionTitle}
        </Text>
        <TouchableOpacity onPress={this.closeModal} style={styles.closeButton}>
          <View style={styles.closeButtonIcon}>
            <View style={styles.closeButtonLeft} />
            <View style={styles.closeButtonRight} />
          </View>
        </TouchableOpacity>
      </View>
    );
  };

  handlePress = (msg: LPInboxMessage) => () => {
    if (msg.data && msg.data['webview-action']) {
      this.handleModalWebView(msg.data['webview-action'], msg.title);
    } else if (msg.data && msg.data['buttonAction']) {
      const parts = msg.data['buttonAction'].split(':');
      const actionName = parts[0];
      const actionUrl = parts.slice(1).join(':');
      switch (actionName) {
        case 'webview-action':
          this.handleModalWebView(actionUrl, msg.title);
          break;
        case 'deeplink-action':
          Linking.openURL(actionUrl);
          break;
        default:
          break;
      }
    }

    msg.read();
  };

  handleModalWebView = (url, title) => {
    this.setState({
      webviewVisible: true,
      webviewActionUrl: url,
      webviewActionTitle: title
    });
  };

  renderMessage = (listItem: ListRenderItemInfo<LPInboxMessage>) => {
    const msg = listItem.item;
    const { width } = Dimensions.get('window');
    const shouldShowButton = !!(msg.data && msg.data['buttonTitle']);

    const content = (
      <View>
        {msg.imageURL ? (
          <AutoHeightImage
            width={width}
            uri={msg.imageURL}
            defaultHeight={100}
          />
        ) : null}

        <View style={styles.messageInfo}>
          <Image style={styles.messageLogoImage} source={this.props.logo} />
          <View style={styles.messageLogoView}>
            <Text style={[styles.fontSize10, this.getFontName()]}>
              {this.props.appName}
            </Text>
            {msg.title ? (
              <Text style={[styles.fontSize14, this.getFontName(), this.getMessageTitleStyle()]}>
                {msg.title}
              </Text>
            ) : null}
            <Text style={[styles.fontSize10, this.getFontName()]}>
              {moment(new Date(msg.deliveryTimestamp)).fromNow()}
            </Text>
          </View>
        </View>

        {msg.subtitle ? (
          <Text style={[styles.messageSubtitle, this.getFontName()]}>
            {msg.subtitle}
          </Text>
        ) : null}
      </View>
    );

    if (shouldShowButton) {
      return (
        <View key={msg.messageId} style={styles.messageContainer}>
          {content}
          <View style={styles.buttonContainer}>
            <TouchableOpacity
              style={[styles.button, this.props.buttonStyle]}
              onPress={this.handlePress(msg)}
            >
              <Text style={[styles.buttonText, this.props.buttonTextStyle]}>
                {msg.data['buttonTitle']}
              </Text>
            </TouchableOpacity>
          </View>
        </View>
      );
    } else {
      return (
        <TouchableOpacity
          key={msg.messageId}
          style={styles.messageContainer}
          onPress={this.handlePress(msg)}
        >
          {content}
        </TouchableOpacity>
      );
    }
  };
}
